# SPDX-FileCopyrightText: Copyright (c) 2022 Alec Delaney for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""
`adafruit_pastebin.adafruit_io`
===============================

Pastebin API for Adafuit IO


* Author(s): Alec Delaney

"""

import json
from adafruit_pastebin import _Pastebin

try:
    import typing  # pylint: disable=unused-import
    from adafruit_pastebin import SupportsStr
    from adafruit_requests import Session
except ImportError:
    pass

POST_URL = "https://io.adafruit.com/api/v2/{username}/feeds/{feed_key}/data"


class AIOPastebin(_Pastebin):
    """
    Pastebin API for Adafruit IO.

    :param Session session: An :py:class:~`adafruit_requests.Session`
        to use for web connectiviy
    :param str auth_key: The Adafruit IO key to use for authentication
    :param str username: The username associated with the ``auth_key``
    :param str feed_key: The feed key of the feed to use for pasting
    """

    def __init__(
        self, session: Session, auth_key: str, *, username: str, feed_key: str
    ) -> None:
        super().__init__(session, auth_key)
        self._username = username
        self._feed_key = feed_key
        self._post_url = (
            f"https://io.adafruit.com/api/v2/{username}/feeds/{feed_key}/data"
        )

    def paste(
        self,
        content: SupportsStr,
    ) -> str:
        """
        Paste content to Adafruit IO and returns the URL of the feed.

        :param content: Any string (or object that can be converted to a string)
            to paste
        """

        payload = {
            "value": str(content),
        }

        headers = {
            "X-AIO-Key": self._auth_key,
        }

        response = self._session.post(self._post_url, json=payload, headers=headers)
        response_json = json.loads(response.text)

        if "error" in response_json:
            raise RuntimeError("Could not paste the information")

        return f"https://io.adafruit.com/{self._username}/feeds/{self._feed_key}"



# SPDX-FileCopyrightText: Copyright (c) 2022 Alec Delaney for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""
`adafruit_pastebin.gist`
========================

Pastebin API for GitHub Gists


* Author(s): Alec Delaney

"""

import json
from adafruit_pastebin import _Pastebin

try:
    from typing import Optional
    from adafruit_pastebin import SupportsStr
except ImportError:
    pass

POST_URL = "https://api.github.com/gists"


class Gist(_Pastebin):
    """
    Pastebin API for GitHub Gists.

    :param Session session: An :py:class:~`adafruit_requests.Session`
        to use for web connectiviy
    :param str auth_key: The `GitHub Personal Access Token
        <https://docs.github.com/en/authentication/keeping-your-account-and-data-secure/creating-a-personal-access-token>`_
        to use for authentication
    """

    def paste(
        self,
        content: SupportsStr,
        *,
        filename: str = "My Gist",
        description: Optional[str] = None,
        public: bool = False,
    ) -> str:
        """
        Paste content as a GitHub Gist and return the URL of the new gist.

        :param content: Any string (or object that can be converted to a string)
            to paste
        :param str filename: (Optional) A filename for the gist content
        :param str|None description: (Optional) A description for the gist
        :param bool public: Whether the gist should be public (``True``) or
            private (``False``)
        """

        headers = {
            "Accept": "applciation/vnd.github+json",
            "Authorization": f"Bearer {self._auth_key}",
        }

        data = {"public": public, "files": {filename: {"content": str(content)}}}

        if description is not None:
            data["description"] = description

        response = self._session.post(POST_URL, headers=headers, data=json.dumps(data))

        json_response = json.loads(response.text)
        try:
            return json_response["html_url"]
        except KeyError:
            error_message = json_response["message"]
        raise RuntimeError(error_message)



# SPDX-FileCopyrightText: 2017 Scott Shawcroft, written for Adafruit Industries
# SPDX-FileCopyrightText: Copyright (c) 2022 Alec Delaney for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""
`adafruit_pastebin`
===================

CircuitPython library for interacting with online pastebin services


* Author(s): Alec Delaney

Implementation Notes
--------------------

**Software and Dependencies:**

* Adafruit CircuitPython firmware for the supported boards:
  https://circuitpython.org/downloads

"""

try:
    from typing import List
    from typing_extensions import Protocol
    from adafruit_requests import Session

    class SupportsStr(Protocol):
        """Protocol type for anything that supports the :py:meth:`str()` method"""

        def __str__(self) -> str:  # pylint: disable=invalid-str-returned
            ...

except ImportError:
    pass

__version__ = "1.0.0"
__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_Pastebin.git"


# pylint: disable=too-few-public-methods
class _Pastebin:
    """
    Generic paste bin class

    TODO: Add more details
    """

    def __init__(self, session: Session, auth_key: str) -> None:
        self._session = session
        self._auth_key = auth_key



# SPDX-FileCopyrightText: Copyright (c) 2022 Alec Delaney for Adafruit Industries
#
# SPDX-License-Identifier: MIT

"""
`adafruit_pastebin.pastebin`
============================

Pastebin API for PasteBin.com


* Author(s): Alec Delaney

"""

from adafruit_pastebin import _Pastebin

try:
    from typing import Optional
    from adafruit_pastebin import SupportsStr  # pylint: disable=ungrouped-imports
except ImportError:
    pass


POST_URL = "http://pastebin.com/api/api_post.php"


# pylint: disable=too-few-public-methods
class PrivacySetting:
    """
    Enum-like class for privacy settings.

    Valid options are:

    * PUBLIC
    * UNLISTED
    * PRIVATE

    """

    PUBLIC = "0"
    UNLISTED = "1"
    PRIVATE = "2"


# pylint: disable=too-few-public-methods
class ExpirationSetting:
    """
    Enum-like class for expiration settings.

    Valid options are:

    * NEVER
    * TEN_MINUTES
    * ONE_HOUR
    * ONE_DAY
    * ONE_WEEK
    * TWO_WEEKS
    * ONE_MONTH
    * SIX_MONTHS
    * ONE_YEAR

    """

    NEVER = "N"
    TEN_MINUTES = "10M"
    ONE_HOUR = "1H"
    ONE_DAY = "1D"
    ONE_WEEK = "1W"
    TWO_WEEKS = "2W"
    ONE_MONTH = "1M"
    SIX_MONTHS = "6M"
    ONE_YEAR = "1Y"


class PasteBin(_Pastebin):
    """
    Pastebin API for PasteBin.com.

    :param Session session: An :py:class:~`adafruit_requests.Session`
        to use for web connectiviy
    :param str auth_key: The `PasteBin.com dev key <https://pastebin.com/doc_api#1>`_
        to use for API accessibilty
    """

    def paste(
        self,
        content: SupportsStr,
        *,
        name: Optional[str] = None,
        content_format: Optional[str] = None,
        privacy: str = PrivacySetting.PUBLIC,
        expiration: str = ExpirationSetting.NEVER,
    ) -> str:
        """
        Paste content to PasteBin.com and return the URL of the new paste.

        :param content: Any string (or object that can be converted to a string)
            to paste
        :param str|None name: (Optional) A name for paste
        :param str|None content_format: (Optional) The formatting of the pasted content;
            valid formats can be found `in the PasteBin.com API docs
            <https://pastebin.com/doc_api#5>`_
        :param str privacy: (Optional) The privacy setting of the paste, which must be a
            valid option from :py:class:`~adafruit_pastebin.pastebin.PrivacySetting`;
            default is public
        :param str expiration: (Optional) When the paste will expire and be deleted,
            which must be a valid option from
            :py:class:`~adafruit_pastebin.pastebin.ExpirationSetting`; default is never
        """

        data = {
            "api_dev_key": self._auth_key,
            "api_option": "paste",
            "api_paste_code": str(content),
            "api_paste_private": privacy,
            "api_paste_expire_date": expiration,
        }
        if name is not None:
            data["api_paste_name"] = name
        if content_format is not None:
            data["api_paste_format"] = content_format

        response = self._session.post(POST_URL, data=data)
        if not response.text.startswith("http"):
            raise RuntimeError(response.text)

        return response.text



# SPDX-FileCopyrightText: Copyright (c) 2022 Alec Delaney for Adafruit Industries
#
# SPDX-License-Identifier: Unlicense

import ssl
import socket
import adafruit_requests as requests
from adafruit_pastebin.adafruit_io import AIOPastebin

try:
    from secrets import secrets
except ImportError:
    print("Please place your auth/dev key in a secrets.py file!")
    raise

auth_key = secrets["auth_key"]

session = requests.Session(socket, ssl_context=ssl.create_default_context())

pastebin = AIOPastebin(
    session, auth_key, username="username", feed_key="existing_feedkey"
)
paste_url = pastebin.paste(
    "This is a test paste!",
)
print(paste_url)
